/**
 * Copyright 2006 (C) Bernd Bruegge, Timo Wolf - TU Muenchen
 * http://wwwbruegge.in.tum.de/Asteroids
 * 
 * This file is part of the Asteroids Teaching Design Pattern Project.
 * 
 * Asteroids is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * Asteroids is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * Asteroids; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package de.tum.in.wwwbruegge.asteroids.control;

import java.applet.AudioClip;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.util.HashSet;

import javax.swing.JOptionPane;

import de.tum.in.wwwbruegge.asteroids.solidbodies.BigAsteroid;
import de.tum.in.wwwbruegge.asteroids.solidbodies.Rocket;
import de.tum.in.wwwbruegge.asteroids.solidbodies.SmallAsteroid;
import de.tum.in.wwwbruegge.asteroids.solidbodies.SolidBody;
import de.tum.in.wwwbruegge.asteroids.solidbodies.SpaceShuttle;
import de.tum.in.wwwbruegge.asteroids.ui.GameBoard;
import de.tum.in.wwwbruegge.asteroids.ui.ToolBar;

/**
 * @author Timo Wolf
 */
public class Referee implements Runnable {

	public static AudioClip MUSIC;

	public static AudioClip BANG;

	private static int SLEEP_TIME = 200;

	private static Referee SINGELTON;

	private Thread theThread;

	private boolean isRunning;

	private long starttime;

	private long gameduration_in_seconds;

	private Referee() {
		setFramesPerSecond(30);
	}

	public static Referee getInstance() {
		if (SINGELTON == null) {
			SINGELTON = new Referee();
		}
		return SINGELTON;
	}

	public void initGame() {
		ToolBar.getInstance().enableButtons();
		GameBoard.getInstance().initiate();
	}

	public void startGame() {
		if (isRunning) {
			return;
		}
		GameBoard.getInstance().initiate();
		MUSIC.loop();
		isRunning = true;
		theThread = new Thread(this);
		theThread.start();
		ToolBar.getInstance().enableButtons();
		this.starttime = System.currentTimeMillis();
		this.gameduration_in_seconds = 0;
	}

	public void stopGame() {
		if (!isRunning) {
			return;
		}
		MUSIC.stop();
		isRunning = false;
		ToolBar.getInstance().enableButtons();
		this.gameduration_in_seconds = (System.currentTimeMillis() - starttime) / 1000;
	}

	public void run() {
		while (isRunning) {
			try {
				Thread.sleep(SLEEP_TIME);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			moveSolidBodies();
		}
	}

	private void moveSolidBodies() {
		GameBoard gameBoard = GameBoard.getInstance();
		SolidBody[] solidbodies = gameBoard.getSolidBodies();

		int max_x = gameBoard.getSize().width;
		int max_y = gameBoard.getSize().height;
		for (int i = 0; i < solidbodies.length; i++) {
			solidbodies[i].updatePosition(max_x, max_y);
		}
		gameBoard.repaint();

		HashSet<SolidBody> crashedBodyCache = new HashSet<SolidBody>();
		for (int z = 0; z < solidbodies.length; z++) {
			SolidBody solidbody1 = solidbodies[z];

			for (int i = 0; i < solidbodies.length; i++) {
				SolidBody solidbody2 = solidbodies[i];
				if (solidbody1 == solidbody2) {
					continue;
				}

				SolidBody crashedBody = detectCollision(solidbody1, solidbody2);
				if (crashedBody != null
						&& !crashedBodyCache.contains(crashedBody)) {
					GameBoard.getInstance().removeSolidBody(crashedBody);
					crashedBodyCache.add(crashedBody);

					if (crashedBody.equals(gameBoard.getSpaceShuttle())) {
						BANG.play();
						stopGame();
						JOptionPane
								.showMessageDialog(null,
										"You lost the game in "
												+ gameduration_in_seconds
												+ " seconds!", "Information",
										JOptionPane.INFORMATION_MESSAGE);
						initGame();
						return;
					} else if (crashedBody instanceof BigAsteroid) {
						for (int j = 0; j < 3; j++) {
							int direction = (int) (Math.random() * 360);
							SmallAsteroid smallasteroid = new SmallAsteroid(
									crashedBody.getPosition(), direction);
							GameBoard.getInstance().addSolidBody(smallasteroid);
						}
					}
				}
			}
		}

		if (GameBoard.getInstance().getSolidBodies().length == 1) {
			stopGame();
			JOptionPane.showMessageDialog(null,
					"Congratulation, you won the game in "
							+ gameduration_in_seconds + " seconds!",
					"Information", JOptionPane.INFORMATION_MESSAGE);
		}
	}

	public SolidBody detectCollision(SolidBody solidbody1, SolidBody solidbody2) {
		Point p1 = GameBoard.getInstance().convertPosition(
				solidbody1.getPosition());
		Dimension d1 = solidbody1.getSize();
		Rectangle r1 = new Rectangle(p1, d1);

		Point p2 = GameBoard.getInstance().convertPosition(
				solidbody2.getPosition());
		Dimension d2 = solidbody2.getSize();
		Rectangle r2 = new Rectangle(p2, d2);

		if (r1.intersects(r2)) {
			if (solidbody1 instanceof SpaceShuttle) {
				return solidbody1;

			} else if (solidbody2 instanceof SpaceShuttle) {
				return solidbody2;

			} else if (solidbody1 instanceof Rocket) {
				return solidbody2;

			} else if (solidbody2 instanceof Rocket) {
				return solidbody1;

			} else {
				return null;
			}
		}
		return null;
	}

	public boolean isRunning() {
		return this.isRunning;
	}

	public static int getFramesPerSecond() {
		return 1000 / SLEEP_TIME;
	}

	public static void setFramesPerSecond(int framesPerSecond)
			throws IllegalArgumentException {
		if (framesPerSecond > 0) {
			SLEEP_TIME = 1000 / framesPerSecond;
		} else {
			throw new IllegalArgumentException(
					"Frames per second must be greater than 0.");
		}
	}
}
