/**
 * Copyright 2006 (C) Bernd Bruegge, Timo Wolf - TU Muenchen
 * http://wwwbruegge.in.tum.de/Asteroids
 * 
 * This file is part of the Asteroids Teaching Design Pattern Project.
 * 
 * Asteroids is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * Asteroids is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * Asteroids; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package de.tum.in.wwwbruegge.asteroids.solidbodies;

import java.awt.Dimension;
import java.awt.Image;
import java.awt.Point;

/**
 * @author Timo Wolf
 */
public abstract class SolidBody {
	public static Image DEFAULT_BODY;

	private int speed;

	private int direction = 90;

	private Image body;

	protected Point position = new Point(5, 5);

	private float delta_x;

	private float delta_y;

	private Dimension size;

	public SolidBody(Point position, int direction) {
		this.position = position;
		setDirection(direction);
		int initialSpeed = (int) (Math.random() * getMaximumSpeed());
		if (initialSpeed < getMinimumSpeed()) {
			initialSpeed = getMinimumSpeed();
		}
		this.speed = initialSpeed;
		setBody(DEFAULT_BODY);
	}

	public int getMaximumSpeed() {
		return 500;
	}

	public int getMinimumSpeed() {
		return 2;
	}

	public Image getBody() {
		return this.body;
	}

	public void setBody(Image body) throws IllegalArgumentException {
		if (body == null) {
			throw new IllegalArgumentException(
					"The chassis image of a car connot be null.");
		}
		this.body = body;
		this.size = new Dimension(this.body.getWidth(null), this.body
				.getHeight(null));
	}

	public Point getPosition() {
		return (Point) this.position.clone();
	}

	public Dimension getSize() {
		return size;
	}

	public synchronized void updatePosition(int max_x, int max_y) {

		position.x = position.x + (int) delta_x;
		position.y = position.y + (int) delta_y;
		if (position.x < 0) {
			position.x = 0;
			setDirection(360 - direction);
		}

		if (position.x + getSize().width > max_x) {
			position.x = max_x - getSize().width;
			setDirection(360 - direction);
		}
		if (position.y - getSize().height < 0) {
			position.y = getSize().height;
			int tmp = 180 - direction;
			if (tmp < 0) {
				setDirection(360 + tmp);
			} else {
				setDirection(tmp);
			}
		}
		if (position.y > max_y) {
			position.y = max_y;
			int tmp = 180 - direction;
			if (tmp < 0) {
				setDirection(360 + tmp);
			} else {
				setDirection(tmp);
			}
		}
	}

	public int getDirection() {
		return this.direction;
	}

	protected void setDirection(int direction) throws IllegalArgumentException {
		if (direction < 0 || direction > 360) {
			throw new IllegalArgumentException(
					"Direction must be between 0 and 360. It was " + direction);
		}
		this.direction = direction;
		updateDeltas();
	}

	protected void setSpeed(int speed) {
		if (speed < getMinimumSpeed() || speed > getMaximumSpeed()) {
			throw new IllegalArgumentException("The speed must be between "
					+ getMinimumSpeed() + " and " + getMaximumSpeed()
					+ ". It was " + direction);
		}
		this.speed = speed;
		updateDeltas();
	}

	public int getSpeed() {
		return speed;
	}

	private void updateDeltas() {
		delta_x = (int) (((float) speed) / 100 * (float) Math.sin(Math
				.toRadians(direction)));
		delta_y = (int) (((float) speed) / 100 * (float) Math.cos(Math
				.toRadians(direction)));
	}
}
